# conf.py to run tests

master_doc = 'index'
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.autosectionlabel',
    'hoverxref.extension',
]

hoverxref_project = 'myproject'
hoverxref_version = 'myversion'
